# Generated by Django 4.2.6 on 2023-10-17 08:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories_editor', '0002_alter_webstory_title'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='webstory',
            options={'ordering': ['first_published_at'], 'verbose_name': 'Web Story', 'verbose_name_plural': 'Web Stories'},
        ),
    ]



import json

from django.contrib.contenttypes.fields import GenericRelation
from django.db import models
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from wagtail.admin.admin_url_finder import AdminURLFinder
from wagtail.admin.panels import FieldPanel, PublishingPanel, InlinePanel
from wagtail.contrib.routable_page.models import RoutablePageMixin, path
from wagtail.contrib.settings.models import BaseSiteSetting
from wagtail.models import (
    DraftStateMixin,
    LockableMixin,
    RevisionMixin,
    WorkflowMixin,
    PreviewableMixin, Page, Orderable
)

from wagtail_webstories_editor import get_webstories_listing_page_model


class WebStoriesSetting(ClusterableModel, BaseSiteSetting):
    google_analytics_id = models.CharField(max_length=255, blank=True, null=True,
                                           verbose_name=_("Google Analytics Measurement ID"))
    using_legacy_analytics = models.BooleanField(default=False)
    video_cache = models.BooleanField(default=False)
    auto_advance = models.BooleanField(default=False)
    default_page_duration = models.IntegerField(default=7, blank=True, null=True)

    panels = [
        FieldPanel("google_analytics_id"),
        FieldPanel("using_legacy_analytics"),
        FieldPanel("video_cache"),
        FieldPanel("auto_advance"),
        FieldPanel("default_page_duration"),
        InlinePanel("publisher_logos", heading=_("Publisher Logos"), label=_("Logo"))
    ]

    @property
    def config(self):
        return {
            "googleAnalyticsId": self.google_analytics_id,
            "usingLegacyAnalytics": self.using_legacy_analytics,
            "videoCache": self.video_cache,
            "autoAdvance": self.auto_advance,
            "defaultPageDuration": self.default_page_duration,
        }


class WebStoriesPublisherLogo(Orderable):
    setting = ParentalKey(WebStoriesSetting, related_name="publisher_logos", on_delete=models.CASCADE)
    logo = models.ForeignKey(
        "wagtailimages.Image",
        null=True,
        blank=False,
        on_delete=models.CASCADE,
        related_name="+",
        verbose_name=_("Logo")
    )
    default = models.BooleanField(default=False)


class WebStory(WorkflowMixin, DraftStateMixin, LockableMixin, RevisionMixin, PreviewableMixin, models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    title = models.CharField(max_length=255, default="Untitled", verbose_name=_("Title"))
    slug = models.CharField(max_length=255, blank=True, null=True, unique=True)
    config = models.JSONField(blank=True, null=True)
    html = models.TextField(blank=True, null=True)

    _revisions = GenericRelation("wagtailcore.Revision", related_query_name="web_story")
    workflow_states = GenericRelation(
        "wagtailcore.WorkflowState",
        content_type_field="base_content_type",
        object_id_field="object_id",
        related_query_name="web_story",
        for_concrete_model=False,
    )

    class Meta:
        verbose_name = _("Web Story")
        verbose_name_plural = _("Web Stories")
        ordering = ["first_published_at", ]

    panels = [
        FieldPanel("title"),
        FieldPanel("slug"),
        FieldPanel("config"),
        FieldPanel("html"),
        PublishingPanel(),
    ]

    def __str__(self):
        return self.title

    @property
    def poster_image_url(self):
        try:
            featured_media = self.config.get("featuredMedia")
            if featured_media and featured_media.get("url"):
                return featured_media.get("url")

            pages = self.config.get("storyData", {}).get("pages")

            for page in pages:
                elements = page.get("elements")
                for element in elements:
                    if element.get("type") == "image":
                        return element.get("resource").get("src")

        except Exception:
            pass

        return None

    def slug_is_available(self, candidate_slug):
        siblings = WebStory.objects.all()

        if self.pk:
            siblings = siblings.exclude(pk=self.pk)

        return not siblings.filter(slug=candidate_slug).exists()

    def full_clean(self, *args, **kwargs):
        candidate_slug = self.slug
        if candidate_slug:
            suffix = 1
            while not self.slug_is_available(candidate_slug):
                suffix += 1
                candidate_slug = "%s-%d" % (self.slug, suffix)
        self.slug = candidate_slug

        super().full_clean(*args, **kwargs)

    @property
    def json_config(self):
        return json.dumps(self.config)

    @property
    def revisions(self):
        return self._revisions

    def get_preview_template(self, request, preview_mode):
        return f"wagtail_webstories_editor/story_detail.html"

    def get_preview_context(self, request, preview_mode):
        return {"story": self}

    def get_parent_link(self, request=None):
        WebStoyListPage = get_webstories_listing_page_model()
        if WebStoyListPage:
            list_page = WebStoyListPage.objects.live().first()
            return list_page.get_full_url(request)
        return None

    def get_link(self, request=None):
        parent_link = self.get_parent_link(request)
        if parent_link:
            suffix = str(self.pk)
            if self.slug:
                suffix = self.slug
            return parent_link + suffix

        return ""

    def get_permalink_template(self, request=None):
        parent_link = self.get_parent_link(request)

        if parent_link:
            return parent_link + "%pagename%/"

        return ""

    def get_story_dashboard_config(self, request=None):
        finder = AdminURLFinder()
        edit_url = finder.get_edit_url(self)

        story = self.get_latest_revision_as_object()

        if request:
            edit_url = request.build_absolute_uri(edit_url)

        created_at = story.last_published_at or story.created_at

        story_data = {
            "id": story.pk,
            "title": self.title,
            "created": created_at.strftime("%Y-%m-%dT%H:%M:%S"),
            "createdGmt": created_at.strftime("%Y-%m-%dT%H:%M:%SZ"),
            "status": "publish" if self.live else "draft",
            "bottomTargetAction": edit_url,
            "editStoryLink": edit_url,
            "capabilities": {
                "hasEditAction": True,
                "hasDeleteAction": False
            },
        }

        try:
            if story.config and story.config.get("featuredMedia"):
                featuredMedia = story.config.get("featuredMedia")
                if featuredMedia and featuredMedia.get("url"):
                    story_data.update({"featuredMediaUrl": featuredMedia.get("url")})
        except Exception:
            pass

        if self.latest_revision:
            story_data.update({
                "modified": self.latest_revision.created_at.strftime("%Y-%m-%dT%H:%M:%SZ")
            })

        link = self.get_link(request)

        if self.live and link:
            story_data.update({
                "link": link,
                "previewLink": link
            })

        return story_data


class AbstractWebStoryListPage(RoutablePageMixin, Page):
    # we should only have one instance of the listing page
    max_count = 1

    class Meta:
        abstract = True

    @property
    def live_stories(self):
        live_webstories_stories = WebStory.objects.filter(live=True).order_by("-last_published_at")
        return live_webstories_stories

    def get_sitemap_urls(self, request=None):
        list_page_sitemap_urls = super(AbstractWebStoryListPage, self).get_sitemap_urls(request)
        list_page_url = self.get_full_url(request=request)

        for story in self.live_stories:
            sitemap_url = {
                "lastmod": story.last_published_at
            }

            if story.slug:
                sitemap_url.update({
                    "location": list_page_url + story.slug
                })
            else:
                sitemap_url.update({
                    "location": list_page_url + str(story.pk)
                })

            list_page_sitemap_urls.append(sitemap_url)

        return list_page_sitemap_urls

    @path('<str:story_id>/')
    def web_story_page(self, request, story_id):
        """
        View function for web story
        """
        try:
            story_id = int(story_id)
            web_story = get_object_or_404(WebStory, live=True, pk=story_id)
        except ValueError:
            web_story = get_object_or_404(WebStory, live=True, slug=story_id)

        return self.render(
            request,
            context_overrides={
                'story': web_story,
            },
            template="wagtail_webstories_editor/story_detail.html"
        )



# Generated by Django 4.2.6 on 2023-10-14 19:09

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import wagtail.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0089_log_entry_data_json_null_to_object'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='WebStory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('live', models.BooleanField(default=True, editable=False, verbose_name='live')),
                ('has_unpublished_changes', models.BooleanField(default=False, editable=False, verbose_name='has unpublished changes')),
                ('first_published_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='first published at')),
                ('last_published_at', models.DateTimeField(editable=False, null=True, verbose_name='last published at')),
                ('go_live_at', models.DateTimeField(blank=True, null=True, verbose_name='go live date/time')),
                ('expire_at', models.DateTimeField(blank=True, null=True, verbose_name='expiry date/time')),
                ('expired', models.BooleanField(default=False, editable=False, verbose_name='expired')),
                ('locked', models.BooleanField(default=False, editable=False, verbose_name='locked')),
                ('locked_at', models.DateTimeField(editable=False, null=True, verbose_name='locked at')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('config', models.JSONField(blank=True, null=True)),
                ('html', models.TextField(blank=True, null=True)),
                ('latest_revision', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.revision', verbose_name='latest revision')),
                ('live_revision', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.revision', verbose_name='live revision')),
                ('locked_by', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='locked_%(class)ss', to=settings.AUTH_USER_MODEL, verbose_name='locked by')),
            ],
            options={
                'verbose_name': 'Web Story',
                'verbose_name_plural': 'Web Stories',
            },
            bases=(wagtail.models.WorkflowMixin, models.Model),
        ),
    ]



# Generated by Django 4.2.6 on 2023-10-20 08:48

from django.db import migrations, models
import django.db.models.deletion
import modelcluster.fields


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0089_log_entry_data_json_null_to_object'),
        ('wagtailimages', '0025_alter_image_file_alter_rendition_file'),
        ('wagtail_webstories_editor', '0004_webstory_created_at'),
    ]

    operations = [
        migrations.CreateModel(
            name='WebStoriesSetting',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.site')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='WebStoriesPublisherLogo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('logo', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailimages.image', verbose_name='Logo')),
                ('setting', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='publisher_logos', to='wagtail_webstories_editor.webstoriessetting')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
    ]



# Generated by Django 4.2.6 on 2023-10-15 10:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories_editor', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='webstory',
            name='title',
            field=models.CharField(default='Untitled', max_length=255, verbose_name='Title'),
        ),
    ]



# Generated by Django 4.2.6 on 2023-10-22 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories_editor', '0007_webstoriessetting_google_analytics_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='webstoriessetting',
            name='auto_advance',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='webstoriessetting',
            name='default_page_duration',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='webstoriessetting',
            name='using_legacy_analytics',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='webstoriessetting',
            name='video_cache',
            field=models.BooleanField(default=False),
        ),
    ]



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def get_webstories_editor_listing_page_model_string():
    """
    Get the dotted ``app.Model`` name for the image model as a string.
    Useful for developers making Wagtail plugins that need to refer to the
    image model, such as in foreign keys, but the model itself is not required.
    """
    return getattr(settings, "WAGTAIL_WEBSTORIES_EDITOR_LISTING_PAGE_MODEL", "")


def get_webstories_listing_page_model():
    """
    Get the webstories listing page model from the ``WAGTAIL_WEBSTORIES_EDITOR_LISTING_PAGE_MODEL`` setting.
    Defaults to None if no model is defined.
    """

    from django.apps import apps

    model_string = get_webstories_editor_listing_page_model_string()

    if not model_string:
        return None

    try:
        return apps.get_model(model_string, require_ready=False)
    except ValueError:
        return None
    except LookupError:
        raise ImproperlyConfigured(
            "WAGTAIL_WEBSTORIES_EDITOR_LISTING_PAGE_MODEL refers to model '%s' that has not been installed"
            % model_string
        )



from django.contrib.admin.utils import quote
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from wagtail.snippets.action_menu import ActionMenuItem
from wagtail.snippets.permissions import get_permission_name


class DeleteMenuItem(ActionMenuItem):
    name = "action-delete"
    label = _("Delete")
    icon_name = "bin"
    
    def __init__(self, order=None):
        super().__init__(order)
    
    def is_shown(self, context):
        delete_permission = get_permission_name("delete", context["model"])
        
        return (
                context["view"] == "edit"
                and context["request"].user.has_perm(delete_permission)
                and not context.get("locked_for_user")
        )
    
    def get_url(self, context):
        instance = context["instance"]
        url_name = instance.snippet_viewset.get_url_name("delete")
        return reverse(url_name, args=[quote(instance.pk)])



# Generated by Django 4.2.6 on 2023-10-22 13:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories_editor', '0005_webstoriessetting_webstoriespublisherlogo'),
    ]

    operations = [
        migrations.AddField(
            model_name='webstoriespublisherlogo',
            name='default',
            field=models.BooleanField(default=False),
        ),
    ]



import mimetypes

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.fields import ReadOnlyField
from wagtail.api.v2.utils import get_full_url
from wagtail.documents.api.v2.serializers import DocumentSerializer
from wagtail.documents.api.v2.views import DocumentsAPIViewSet
from wagtail.images.api.v2.serializers import ImageSerializer
from wagtail.images.api.v2.views import ImagesAPIViewSet
from wagtailmedia.api.serializers import MediaItemSerializer
from wagtailmedia.api.views import MediaAPIViewSet


class ImageDownloadUrlField(ReadOnlyField):
    """
    Serializes the "download_url" field for image items.

    Example:
    "download_url": "http://api.example.com/media/my_video.mp4"
    """

    def get_attribute(self, instance):
        return instance

    def to_representation(self, instance):
        return get_full_url(self.context["request"], instance.file.url)


class MimeTypeField(ReadOnlyField):
    """
    Serializes the "mime_type" field for image items.

    Example:
    "mime_type": "image/png"
    """

    def get_attribute(self, instance):
        return instance

    def to_representation(self, instance):
        mimetype, encoding = mimetypes.guess_type(instance.file.name)
        return mimetype


class ImageSizeField(ReadOnlyField):
    """
     Serializes the "size" field for image items.

     Example:
     "size": {width:100, height:100}
     """

    def get_attribute(self, instance):
        return instance

    def to_representation(self, instance):
        width = instance.file.width
        height = instance.file.height
        return {"width": width, "height": height}


class CustomImageSerializer(ImageSerializer):
    download_url = ImageDownloadUrlField()
    mime_type = MimeTypeField()
    size = ImageSizeField()


class CustomImagesAPIViewSet(ImagesAPIViewSet):
    base_serializer_class = CustomImageSerializer
    meta_fields = ImagesAPIViewSet.meta_fields + ["mime_type", "size"]
    listing_default_fields = ImagesAPIViewSet.listing_default_fields + [
        "mime_type", "size"
    ]
    nested_default_fields = ImagesAPIViewSet.nested_default_fields + [
        "mime_type", "size"
    ]


class CustomMediaItemSerializer(MediaItemSerializer):
    mime_type = MimeTypeField()


class CustomMediaAPIViewSet(MediaAPIViewSet):
    base_serializer_class = CustomMediaItemSerializer
    meta_fields = ImagesAPIViewSet.meta_fields + ["mime_type"]
    listing_default_fields = ImagesAPIViewSet.listing_default_fields + ["mime_type"]

    filter_backends = MediaAPIViewSet.filter_backends + [DjangoFilterBackend]
    filterset_fields = ["type"]


class CustomDocumentSerializer(DocumentSerializer):
    mime_type = MimeTypeField()


class CustomDocumentAPIViewSet(DocumentsAPIViewSet):
    base_serializer_class = CustomDocumentSerializer
    meta_fields = ImagesAPIViewSet.meta_fields + ["mime_type"]
    listing_default_fields = ImagesAPIViewSet.listing_default_fields + ["mime_type"]



import json

from django.urls import path, reverse_lazy
from wagtail import hooks
from wagtail.log_actions import log
from wagtail.snippets.models import register_snippet
from wagtail.snippets.views.snippets import SnippetViewSet, EditView, CreateView

from .action_menu import DeleteMenuItem
from .models import WebStory, WebStoriesSetting
from .utils import add_video_cache, add_google_analytics
from .views import (
    web_stories_list,
    update_webstory,
    duplicate_webstory,
    handle_publisher_logos, handle_webstories_settings
)

WEBSTORIES_ICONS = [
    "webstories.svg",
    "webstories-logo-bw.svg",
    "webstories-logo-color.svg",
    "webstories-logo-with-type-bw.svg",
    "webstories-logo-with-type-circle-bw.svg",
    "webstories-logo-with-type-circle-color.svg",
]


@hooks.register("register_admin_urls")
def register_admin_urls():
    return [
        path("web-stories-list/", web_stories_list, name="web_stories_list"),
        path("web-stories-update/<int:story_id>/", update_webstory, name="web_stories_update"),
        path("web-stories-duplicate/<int:story_id>/", duplicate_webstory, name="web_stories_duplicate"),
        path("web-stories-publisher-logos/", handle_publisher_logos, name="web_stories_publisher_logos"),
        path("web-stories-settings/", handle_webstories_settings, name="web_stories_settings"),
    ]


@hooks.register("register_icons")
def register_icons(icons):
    for icon in WEBSTORIES_ICONS:
        icons.append("wagtail_webstories_editor/icons/{}".format(icon))
    return icons


class WebStoryVideoCacheCreateEditMixin:
    def get_context_data(self, **kwargs):
        web_stories_setting = WebStoriesSetting.for_request(self.request)
        
        context = super().get_context_data(**kwargs)
        
        config = {
            "globalAutoAdvance": web_stories_setting.auto_advance,
            "globalPageDuration": web_stories_setting.default_page_duration,
        }
        
        context.update({"editorConfig": json.dumps(config)})
        
        context.update({
            "documentChooserUrl": reverse_lazy("wagtaildocs_chooser:choose"),
            "imageChooserUrl": reverse_lazy("wagtailimages_chooser:choose"),
        })
        
        return context
    
    def save_instance(self):
        """
        Called after the form is successfully validated - saves the object to the db
        and returns the new object. Override this to implement custom save logic.
        """
        
        web_stories_setting = WebStoriesSetting.for_request(self.request)
        
        if self.draftstate_enabled:
            instance = self.form.save(commit=False)
            
            if web_stories_setting.video_cache and instance.html:
                html = add_video_cache(instance.html, web_stories_setting.video_cache)
                instance.html = html
            
            if web_stories_setting.google_analytics_id:
                html = add_google_analytics(instance.html, web_stories_setting.google_analytics_id)
                instance.html = html
            
            # If DraftStateMixin is applied, only save to the database in CreateView,
            # and make sure the live field is set to False.
            if self.view_name == "create":
                instance.live = False
                instance.save()
                self.form.save_m2m()
        else:
            instance = self.form.save(commit=False)
            
            if web_stories_setting.video_cache and instance.html:
                html = add_video_cache(instance.html, web_stories_setting.video_cache)
                instance.html = html
            
            if web_stories_setting.google_analytics_id:
                html = add_google_analytics(instance.html, web_stories_setting.google_analytics_id)
                instance.html = html
            
            instance.save()
        
        self.has_content_changes = self.view_name == "create" or self.form.has_changed()
        
        # Save revision if the model inherits from RevisionMixin
        self.new_revision = None
        if self.revision_enabled:
            self.new_revision = instance.save_revision(user=self.request.user)
        
        log(
            instance=instance,
            action="wagtail.create" if self.view_name == "create" else "wagtail.edit",
            revision=self.new_revision,
            content_changed=self.has_content_changes,
        )
        
        return instance


class WebStoryCreateView(WebStoryVideoCacheCreateEditMixin, CreateView):
    pass


class WebStoryEditView(WebStoryVideoCacheCreateEditMixin, EditView):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        
        link = self.object.get_link(self.request)
        permalink_template = self.object.get_permalink_template(self.request)
        
        if link:
            context.update({"story_link": link})
        
        if permalink_template:
            context.update({"permalink_template": permalink_template})
        
        return context


class WebStoryViewSet(SnippetViewSet):
    model = WebStory
    create_template_name = "wagtail_webstories_editor/create_webstory.html"
    edit_template_name = "wagtail_webstories_editor/edit_webstory.html"
    index_template_name = "wagtail_webstories_editor/index_webstory.html"
    
    add_view_class = WebStoryCreateView
    edit_view_class = WebStoryEditView
    
    icon = "webstories"
    menu_label = "Web Stories"
    menu_name = "web_stories"
    menu_order = 900
    add_to_admin_menu = True


register_snippet(WebStoryViewSet)


@hooks.register('register_snippet_action_menu_item')
def register_delete_menu_item(model):
    # add delete to the menu
    if issubclass(model, WebStory):
        return DeleteMenuItem(order=70)



import json

from bs4 import BeautifulSoup


def add_video_cache(doc, cache_enabled):
    if not cache_enabled:
        return doc

    soup = BeautifulSoup(doc, 'html.parser')
    amp_videos = soup.findAll('amp-video')

    if amp_videos:
        for amp_video in amp_videos:
            amp_video.attrs["cache"] = "google"

    return str(soup)


def add_google_analytics(doc, google_analytics_id):
    default_config = get_default_google_analytics_config(google_analytics_id)

    amp_analytics_script = '''
    <script async="" custom-element="amp-analytics" src="https://cdn.ampproject.org/v0/amp-analytics-0.1.js"></script>
    '''

    amp_analytics_config = f'''
    <amp-analytics type="gtag" data-credentials="include">
      <script type="application/json">
        {json.dumps(default_config)}
      </script>
    </amp-analytics>
    '''

    soup = BeautifulSoup(doc, 'html.parser')

    head_tag = soup.find('head')
    script_tag = head_tag.find('script', src="https://cdn.ampproject.org/v0.js")

    if head_tag and script_tag:
        # Insert the amp-analytics configuration just before the <script> tag
        script_tag.insert_before(BeautifulSoup(amp_analytics_script, 'html.parser'))
        soup.body.insert(1, BeautifulSoup(amp_analytics_config, 'html.parser'))
        return str(soup)

    return doc


def get_default_google_analytics_config(google_analytics_id):
    config = {
        "vars": {
            "gtag_id": google_analytics_id,
            "config": {
                google_analytics_id: {
                    "groups": "default"
                }
            }
        },
        "triggers": {
            # Fired when a story page becomes visible.
            "storyProgress": {
                "on": "story-page-visible",
                "request": "event",
                "vars": {
                    "event_name": "custom",
                    "event_action": "story_progress",
                    "event_category": "${title}",
                    "event_label": "${storyPageIndex}",
                    "event_value": "${storyProgress}",
                    "send_to": google_analytics_id
                }
            },
            # Fired when the last page in the story is shown to the user.
            # This can be used to measure completion rate.
            "storyEnd": {
                "on": "story-last-page-visible",
                "request": "event",
                "vars": {
                    "event_name": "custom",
                    "event_action": "story_complete",
                    "event_category": "${title}",
                    "event_label": "${storyPageCount}",
                    "send_to": google_analytics_id
                }
            },
            # Fired when clicking an element that opens a tooltip (<a> or <amp-twitter>).
            "trackFocusState": {
                "on": "story-focus",
                "tagName": "a",
                "request": "click",
                "vars": {
                    "event_name": "custom",
                    "event_action": "story_focus",
                    "event_category": "${title}",
                    "send_to": google_analytics_id
                }
            },
            # Fired when clicking on a tooltip.
            "trackClickThrough": {
                "on": "story-click-through",
                "tagName": "a",
                "request": "click",
                "vars": {
                    "event_name": "custom",
                    "event_action": "story_click_through",
                    "event_category": "${title}",
                    "send_to": google_analytics_id
                }
            },
            # Fired when opening a drawer or dialog inside a story (e.g. page attachment).
            "storyOpen": {
                "on": "story-open",
                "request": "event",
                "vars": {
                    "event_name": "custom",
                    "event_action": "story_open",
                    "event_category": "${title}",
                    "send_to": google_analytics_id
                }
            },
            # Fired when closing a drawer or dialog inside a story (e.g. page attachment).
            "storyClose": {
                "on": "story-close",
                "request": "event",
                "vars": {
                    "event_name": "custom",
                    "event_action": "story_close",
                    "event_category": "${title}",
                    "send_to": google_analytics_id
                }
            },
            # Fired when the user initiates an interaction to mute the audio for the current story.
            "audioMuted": {
                "on": "story-audio-muted",
                "request": "event",
                "vars": {
                    "event_name": "custom",
                    "event_action": "story_audio_muted",
                    "event_category": "${title}",
                    "send_to": google_analytics_id
                }
            },
            # Fired when the user initiates an interaction to unmute the audio for the current story.
            "audioUnmuted": {
                "on": "story-audio-unmuted",
                "request": "event",
                "vars": {
                    "event_name": "custom",
                    "event_action": "story_audio_unmuted",
                    "event_category": "${title}",
                    "send_to": google_analytics_id
                }
            },
            # Fired when a page attachment is opened by the user.
            "pageAttachmentEnter": {
                "on": "story-page-attachment-enter",
                "request": "event",
                "vars": {
                    "event_name": "custom",
                    "event_action": "story_page_attachment_enter",
                    "event_category": "${title}",
                    "send_to": google_analytics_id
                }
            },
            # Fired when a page attachment is dismissed by the user.
            "pageAttachmentExit": {
                "on": "story-page-attachment-exit",
                "request": "event",
                "vars": {
                    "event_name": "custom",
                    "event_action": "story_page_attachment_exit",
                    "event_category": "${title}",
                    "send_to": google_analytics_id
                }
            }
        }
    }

    return config



# Generated by Django 4.2.6 on 2023-10-17 09:48

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories_editor', '0003_alter_webstory_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='webstory',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]



import json

from django.core.paginator import Paginator
from django.http import JsonResponse
from django.shortcuts import get_object_or_404
from wagtail.api.v2.utils import get_full_url

from wagtail_webstories_editor.models import WebStory, WebStoriesSetting, WebStoriesPublisherLogo


def web_stories_list(request):
    web_stories = WebStory.objects.all().order_by("-last_published_at")
    paginator = Paginator(web_stories, 20)

    stories_data = {
        "stories": {},
        "fetchedStoryIds": [],
        "totalPages": paginator.num_pages,
        "totalStoriesByStatus": {
            "all": paginator.count,
            "publish": web_stories.filter(live=True).count(),
        },
    }

    page_number = request.GET.get("page")
    page_obj = paginator.get_page(page_number)

    for story_obj in page_obj:
        latest_revision = story_obj.get_latest_revision_as_object()
        story_data = latest_revision.get_story_dashboard_config(request)

        stories_data["stories"].update({latest_revision.pk: story_data})
        stories_data["fetchedStoryIds"].append(latest_revision.pk)

    return JsonResponse(stories_data)


def update_webstory(request, story_id):
    web_story = get_object_or_404(WebStory, pk=story_id)

    if request.method == 'POST':
        try:
            data = json.loads(request.body)
            title = data.get("title")

            if title:
                web_story.title = title
                config = web_story.get_latest_revision_as_object().config
                if config:
                    config["title"].update({"raw": title})
                web_story.config = config
                web_story.save_revision(changed=True, log_action=True)

        except Exception:
            pass

    latest_revision = web_story.get_latest_revision_as_object()
    story_config = latest_revision.get_story_dashboard_config(request)

    return JsonResponse(story_config)


def duplicate_webstory(request, story_id):
    web_story = get_object_or_404(WebStory, pk=story_id)

    latest_revision = web_story.get_latest_revision_as_object()

    title = latest_revision.title or "Untitled"

    new_story_data = {
        "title": title + " - Copy",
        "config": latest_revision.config,
        "html": latest_revision.html,
        "live": False
    }

    web_story_copy = WebStory(**new_story_data)
    web_story_copy.save()
    web_story_copy.save_revision(changed=True, log_action=True)

    latest_revision = web_story_copy.get_latest_revision_as_object()

    story_config = latest_revision.get_story_dashboard_config(request)

    return JsonResponse(story_config)


def handle_publisher_logos(request):
    web_stories_setting = WebStoriesSetting.for_request(request)

    if request.method == 'POST':
        logo_res = {}

        try:
            data = json.loads(request.body)
            default_logo_id = data.get("default_logo_id")
            image_id = data.get("id")

            if default_logo_id:
                logo_to_set = web_stories_setting.publisher_logos.filter(logo_id=default_logo_id).first()

                for p_logo in web_stories_setting.publisher_logos.all():
                    if p_logo.pk == logo_to_set.pk:
                        p_logo.default = True
                    else:
                        p_logo.default = False

                    p_logo.save()

                logo_res.update({
                    "id": logo_to_set.logo.id,
                    "url": get_full_url(request, logo_to_set.logo.file.url)
                })
            else:
                logo = WebStoriesPublisherLogo(logo_id=image_id)
                web_stories_setting.publisher_logos.add(logo)
                web_stories_setting.save()

                logo_res.update({
                    "id": logo.logo.id,
                    "url": get_full_url(request, logo.logo.file.url)
                })
        except Exception:
            pass

        return JsonResponse(logo_res)

    publisher_logos = web_stories_setting.publisher_logos.all()
    logos = []
    for logo in publisher_logos:
        logos.append({
            "id": logo.logo.id,
            "url": get_full_url(request, logo.logo.file.url),
            "active": logo.default,
        })

    return JsonResponse(logos, safe=False)


def handle_webstories_settings(request):
    web_stories_setting = WebStoriesSetting.for_request(request)

    if request.method == "POST":
        try:
            data = json.loads(request.body)

            for key, value in data.items():
                setattr(web_stories_setting, key, value)

            web_stories_setting.save()

        except Exception:
            pass

    res = web_stories_setting.config

    return JsonResponse(res)



# Generated by Django 4.2.6 on 2023-10-22 14:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories_editor', '0006_webstoriespublisherlogo_default'),
    ]

    operations = [
        migrations.AddField(
            model_name='webstoriessetting',
            name='google_analytics_id',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Google Analytics Measurement ID'),
        ),
    ]



# Generated by Django 4.2.6 on 2023-10-22 14:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories_editor', '0008_webstoriessetting_auto_advance_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='webstoriessetting',
            name='default_page_duration',
            field=models.IntegerField(blank=True, default=7, null=True),
        ),
    ]



# Generated by Django 4.2.6 on 2023-10-23 06:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories_editor', '0011_remove_webstoriespublisherlogo_slug_webstory_slug'),
    ]

    operations = [
        migrations.AlterField(
            model_name='webstory',
            name='slug',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True),
        ),
    ]



# Generated by Django 4.2.6 on 2023-10-23 06:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories_editor', '0009_alter_webstoriessetting_default_page_duration'),
    ]

    operations = [
        migrations.AddField(
            model_name='webstoriespublisherlogo',
            name='slug',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
